@@ -52,11 +52,17 @@ module Linguist
     # Return true or false
     def generated?
       xcode_file? ||
+      cocoapods? ||
+      carthage_build? ||
+      generated_graphql_relay? ||
       generated_net_designer_file? ||
       generated_net_specflow_feature_file? ||
       composer_lock? ||
+      cargo_lock? ||
       node_modules? ||
       go_vendor? ||
+      go_lock? ||
+      esy_lock? ||
       npm_shrinkwrap_or_package_lock? ||
       godeps? ||
       generated_by_zephir? ||
@@ -68,6 +74,7 @@ module Linguist
       generated_net_docfile? ||
       generated_postscript? ||
       compiled_cython_file? ||
+      pipenv_lock? ||
       generated_go? ||
       generated_protocol_buffer? ||
       generated_javascript_protocol_buffer? ||
@@ -80,9 +87,13 @@ module Linguist
       generated_jflex? ||
       generated_grammarkit? ||
       generated_roxygen2? ||
+      generated_html? ||
       generated_jison? ||
-      generated_yarn_lock? ||
-      generated_grpc_cpp?
+      generated_grpc_cpp? ||
+      generated_dart? ||
+      generated_perl_ppport_header? ||
+      generated_gamemakerstudio? ||
+      generated_gimp?
     end
 
     # Internal: Is the blob an Xcode file?
@@ -90,12 +101,33 @@ module Linguist
     # Generated if the file extension is an Xcode
     # file extension.
     #
-    # Returns true of false.
+    # Returns true or false.
     def xcode_file?
       ['.nib', '.xcworkspacedata', '.xcuserstate'].include?(extname)
     end
 
-    # Internal: Is the blob minified files?
+    # Internal: Is the blob part of Pods/, which contains dependencies not meant for humans in pull requests.
+    #
+    # Returns true or false.
+    def cocoapods?
+      !!name.match(/(^Pods|\/Pods)\//)
+    end
+
+    # Internal: Is the blob part of Carthage/Build/, which contains dependencies not meant for humans in pull requests.
+    #
+    # Returns true or false.
+    def carthage_build?
+      !!name.match(/(^|\/)Carthage\/Build\//)
+    end
+
+    # Internal: Does extname indicate a filetype which is commonly minified?
+    #
+    # Returns true or false.
+    def maybe_minified?
+      ['.js', '.css'].include? extname.downcase
+    end
+
+    # Internal: Is the blob a minified file?
     #
     # Consider a file minified if the average line length is
     # greater then 110c.
@@ -104,32 +136,29 @@ module Linguist
     #
     # Returns true or false.
     def minified_files?
-      return unless ['.js', '.css'].include? extname
-      if lines.any?
+      if maybe_minified? and lines.any?
         (lines.inject(0) { |n, l| n += l.length } / lines.length) > 110
       else
         false
       end
     end
 
-    # Internal: Does the blob contain a source map reference?
+    # Internal: Does the blob contain a source-map reference?
     #
-    # We assume that if one of the last 2 lines starts with a source map
+    # We assume that if one of the last 2 lines starts with a source-map
     # reference, then the current file was generated from other files.
     #
     # We use the last 2 lines because the last line might be empty.
     #
-    # We only handle JavaScript, no CSS support yet.
-    #
     # Returns true or false.
     def has_source_map?
-      return false unless extname.downcase == '.js'
-      lines.last(2).any? { |line| line.start_with?('//# sourceMappingURL') }
+      return false unless maybe_minified?
+      lines.last(2).any? { |l| l.match(/^\/[*\/][\#@] source(?:Mapping)?URL|sourceURL=/) }
     end
 
-    # Internal: Is the blob a generated source map?
+    # Internal: Is the blob a generated source-map?
     #
-    # Source Maps usually have .css.map or .js.map extensions. In case they
+    # Source-maps usually have .css.map or .js.map extensions. In case they
     # are not following the name convention, detect them based on the content.
     #
     # Returns true or false.
@@ -206,7 +235,7 @@ module Linguist
     #
     # Returns true or false
     def generated_net_designer_file?
-      name.downcase =~ /\.designer\.cs$/
+      name.downcase =~ /\.designer\.(cs|vb)$/
     end
 
     # Internal: Is this a codegen file for Specflow feature file?
@@ -273,12 +302,12 @@ module Linguist
       return lines[0].include?("Code generated by")
     end
 
-    PROTOBUF_EXTENSIONS = ['.py', '.java', '.h', '.cc', '.cpp']
+    PROTOBUF_EXTENSIONS = ['.py', '.java', '.h', '.cc', '.cpp', '.m', '.rb']
 
     # Internal: Is the blob a C++, Java or Python source file generated by the
     # Protocol Buffer compiler?
     #
-    # Returns true of false.
+    # Returns true or false.
     def generated_protocol_buffer?
       return false unless PROTOBUF_EXTENSIONS.include?(extname)
       return false unless lines.count > 1
@@ -289,7 +318,7 @@ module Linguist
     # Internal: Is the blob a Javascript source file generated by the
     # Protocol Buffer compiler?
     #
-    # Returns true of false.
+    # Returns true or false.
     def generated_javascript_protocol_buffer?
       return false unless extname == ".js"
       return false unless lines.count > 6
@@ -309,7 +338,7 @@ module Linguist
 
     # Internal: Is the blob a C/C++ header generated by the Java JNI tool javah?
     #
-    # Returns true of false.
+    # Returns true or false.
     def generated_jni_header?
       return false unless extname == '.h'
       return false unless lines.count > 2
@@ -333,6 +362,20 @@ module Linguist
       !!name.match(/vendor\/((?!-)[-0-9A-Za-z]+(?<!-)\.)+(com|edu|gov|in|me|net|org|fm|io)/)
     end
 
+    # Internal: Is the blob a generated Go dep or glide lock file?
+    #
+    # Returns true or false.
+    def go_lock?
+      !!name.match(/(Gopkg|glide)\.lock/)
+    end
+
+    # Internal: Is the blob a generated esy lock file?
+    #
+    # Returns true or false.
+    def esy_lock?
+      !!name.match(/(^|\/)(\w+\.)?esy.lock$/)
+    end
+
     # Internal: Is the blob a generated npm shrinkwrap or package lock file?
     #
     # Returns true or false.
@@ -362,6 +405,13 @@ module Linguist
       !!name.match(/.\.zep\.(?:c|h|php)$/)
     end
 
+    # Internal: Is the blob a generated Rust Cargo lock file?
+    #
+    # Returns true or false.
+    def cargo_lock?
+      !!name.match(/Cargo\.lock/)
+    end
+
     # Is the blob a VCR Cassette file?
     #
     # Returns true or false
@@ -385,6 +435,13 @@ module Linguist
       return lines[0].include?("Generated by Cython")
     end
 
+    # Internal: Is this a Pipenv lock file?
+    #
+    # Returns true or false.
+    def pipenv_lock?
+      name.match(/Pipfile\.lock/)
+    end
+
     # Internal: Is it a KiCAD or GFortran module file?
     #
     # KiCAD module files contain:
@@ -395,7 +452,7 @@ module Linguist
     # GFORTRAN module version 'x' created from
     # on the first line.
     #
-    # Return true of false
+    # Return true or false
     def generated_module?
       return false unless extname == '.mod'
       return false unless lines.count > 1
@@ -488,15 +545,6 @@ module Linguist
              lines[0].start_with?("/* generated by jison-lex ")
     end
 
-    # Internal: Is the blob a generated yarn lockfile?
-    #
-    # Returns true or false.
-    def generated_yarn_lock?
-      return false unless name.match(/yarn\.lock/)
-      return false unless lines.count > 0
-      return lines[0].include?("# THIS IS AN AUTOGENERATED FILE")
-    end
-
     # Internal: Is this a protobuf/grpc-generated C++ file?
     #
     # A generated file contains:
@@ -509,5 +557,129 @@ module Linguist
       return false unless lines.count > 1
       return lines[0].start_with?("// Generated by the gRPC")
     end
+
+    # Internal: Is this a generated Dart file?
+    #
+    # A dart-lang/appengine generated file contains:
+    # // Generated code. Do not modify.
+    # on the first line.
+    #
+    # An owl generated file contains:
+    # // GENERATED CODE - DO NOT MODIFY
+    # on the first line.
+    #
+    # Return true or false
+    def generated_dart?
+      return false unless extname == '.dart'
+      return false unless lines.count > 1
+      return lines.first.downcase =~ /generated code\W{2,3}do not modify/
+    end
+
+    # Internal: Is the file a generated Perl/Pollution/Portability header file?
+    #
+    # Returns true or false.
+    def generated_perl_ppport_header?
+        return false unless name.match(/ppport\.h$/)
+        return false unless lines.count > 10
+        return lines[8].include?("Automatically created by Devel::PPPort")
+    end
+
+    # Internal: Is this a relay-compiler generated graphql file?
+    #
+    # Return true or false
+    def generated_graphql_relay?
+      !!name.match(/__generated__\//)
+    end
+
+    # Internal: Is this a generated Game Maker Studio (2) metadata file?
+    #
+    # All Game Maker Studio 2 generated files will be JSON, .yy or .yyp, and have
+    # a part that looks like "modelName: GMname" on the 3rd line
+    #
+    # Return true or false
+    def generated_gamemakerstudio?
+      return false unless ['.yy', '.yyp'].include? extname
+      return false unless lines.count > 3
+      return lines[2].match(/\"modelName\"\:\s*\"GM/) ||
+             lines[0] =~ /^\d\.\d\.\d.+\|\{/
+    end
+        
+    # Internal: Is this a generated GIMP C image file?
+    #
+    # GIMP saves C sources with one of two comment forms:
+    # * `/* GIMP RGB C-Source image dump (<filename>.c) */` (C source export)
+    # * `/*  GIMP header image file format (RGB): <filename>.h  */` (Header export)
+    #
+    # Return true or false
+    def generated_gimp?
+      return false unless ['.c', '.h'].include? extname
+      return false unless lines.count > 0
+      return lines[0].match(/\/\* GIMP [a-zA-Z0-9\- ]+ C\-Source image dump \(.+?\.c\) \*\//) ||
+             lines[0].match(/\/\*  GIMP header image file format \([a-zA-Z0-9\- ]+\)\: .+?\.h  \*\//)
+    end
+
+    # Internal: Is this a generated HTML file?
+    #
+    # HTML documents generated by authoring tools often include a
+    # a <meta> tag in the header of the form:
+    #
+    #    <meta name="generator" content="DocGen v5.0.1" />
+    #
+    # Return true or false
+    def generated_html?
+      return false unless ['.html', '.htm', '.xhtml'].include? extname.downcase
+      return false unless lines.count > 1
+
+      # Pkgdown
+      return true if lines[0..1].any? do |line|
+        line.match(/<!-- Generated by pkgdown: do not edit by hand -->/)
+      end
+
+      # Mandoc
+      return true if lines.count > 2 && lines[2].start_with?('<!-- This is an automatically generated file.')
+
+      # Doxygen
+      return true if lines[0..30].any? do |line|
+        line.match(/<!--\s+Generated by Doxygen\s+[.0-9]+\s*-->/i)
+      end
+
+      # HTML tag: <meta name="generator" content="…" />
+      matches = lines[0..30].join(' ').scan(/<meta(\s+[^>]++)>/i)
+      return false if matches.empty?
+      return matches.map {|x| extract_html_meta(x) }.any? do |attr|
+        attr["name"].to_s.downcase == 'generator' &&
+        [attr["content"], attr["value"]].any? do |cv|
+          !cv.nil? &&
+          cv.match(/^
+            ( org \s+ mode
+            | j?latex2html
+            | groff
+            | makeinfo
+            | texi2html
+            | ronn
+            ) \b
+          /ix)
+        end
+      end
+    end
+
+    # Internal: Extract a Hash of name/content pairs from an HTML <meta> tag
+    def extract_html_meta(match)
+      (match.last.sub(/\/\Z/, "").strip.scan(/
+        (?<=^|\s)              # Check for preceding whitespace
+        (name|content|value)   # Attribute names we're interested in
+        \s* = \s*              # Key-value separator
+
+        # Attribute value
+        ( "[^"]+"        # name="value"
+        | '[^']+'        # name='value'
+        |  [^\s"']+      # name=value
+        )
+      /ix)).map do |match|
+        key = match[0].downcase
+        val = match[1].gsub(/\A["']|["']\Z/, '')
+        [key, val]
+      end.select { |x| x.length == 2 }.to_h
+    end
   end
 end
